From 9a7e7a41c2a69d563c01ecaffdebb8298686874d Mon Sep 17 00:00:00 2001
From: Nikodemus Siivola <nikodemus@random-state.net>
Date: Tue, 29 Nov 2011 10:37:51 +0200
Subject: [PATCH] exact(?) truncation for frame locals

---
 slime.el   |    3 ++-
 swank.lisp |   22 ++++++++++++----------
 2 files changed, 14 insertions(+), 11 deletions(-)

diff --git a/slime.el b/slime.el
index 951a83f..a3b57eb 100644
--- a/slime.el
+++ b/slime.el
@@ -5852,7 +5852,8 @@ The details include local variable bindings and CATCH-tags."
          (num (car frame)))
     (destructuring-bind (start end) (sldb-frame-region)
       (list* start end frame 
-             (slime-eval `(swank:frame-locals-and-catch-tags ,num))))))
+             (slime-eval `(swank:frame-locals-and-catch-tags
+                           ,num ,(window-width (selected-window))))))))
 
 (defvar sldb-insert-frame-variable-value-function 
   'sldb-insert-frame-variable-value)
diff --git a/swank.lisp b/swank.lisp
index 43a6dce..e6f7b7e 100644
--- a/swank.lisp
+++ b/swank.lisp
@@ -2252,11 +2252,12 @@ aborted and return immediately with the output written so far."
             (with-string-stream (,var :length ,length)
               . ,body)))))
 
-(defun to-line  (object &optional (width 75))
+(defun to-line  (object &optional width)
   "Print OBJECT to a single line. Return the string."
-  (without-printing-errors (:object object :stream nil)
-    (with-string-stream (stream :length width)
-      (write object :stream stream :right-margin width :lines 1))))
+  (let ((width (or width 75)))
+    (without-printing-errors (:object object :stream nil)
+      (with-string-stream (stream :length width)
+        (write object :stream stream :right-margin width :lines 1)))))
 
 (defun escape-string (string stream &key length (map '((#\" . "\\\"")
                                                        (#\\ . "\\\\"))))
@@ -2665,20 +2666,21 @@ has changed, ignore the request."
     (cond (pkg (package-name pkg))
           (t (with-buffer-syntax () (package-name *package*))))))
 
-(defslimefun frame-locals-and-catch-tags (index)
+(defslimefun frame-locals-and-catch-tags (index &optional width)
   "Return a list (LOCALS TAGS) for vars and catch tags in the frame INDEX.
 LOCALS is a list of the form ((&key NAME ID VALUE) ...).
 TAGS has is a list of strings."
-  (list (frame-locals-for-emacs index)
+  (list (frame-locals-for-emacs index width)
         (mapcar #'to-string (frame-catch-tags index))))
 
-(defun frame-locals-for-emacs (index)
+(defun frame-locals-for-emacs (index width)
   (with-bindings *backtrace-printer-bindings*
     (loop for var in (frame-locals index)
           collect (destructuring-bind (&key name id value) var
-                    (list :name (prin1-to-string name) 
-                          :id id
-                          :value (to-line value))))))
+                    (let ((name (prin1-to-string name) ))
+                      (list :name name
+                            :id id
+                            :value (to-line value (- width (length name) 13))))))))
 
 (defslimefun sldb-disassemble (index)
   (with-output-to-string (*standard-output*)
-- 
1.7.5.GIT

